import numpy as np

import torch.nn as nn
from torch.optim import Adam
from torch.optim import SGD
import torch.nn.functional as F
import math

import torch

# The machine learning paradigm.
def main():
	data = loadData()
	model = LogRegressionModel()
	optimize(model, data)

# 1) define the model you are optimizing over...
# This is a component of a neural net or any 
# parameterized function you are optimizing over. 
class LogRegressionModel(nn.Module):
	# the initialize method
	def __init__(self):
		super().__init__() # necessary

		# define any parameters are part of the model
		initial1 = torch.ones(1) * 0
		initial2 = torch.ones(1) * 0
		self.theta1 = nn.Parameter(initial1)
		self.theta0 = nn.Parameter(initial2)

	# this what happens when you apply the function
	# to input. In this case I have implemented
	# y = sigma(self.theta1 * x + self.theta0)
	def forward(self, x):
		a = self.theta1 * x + self.theta0
		return torch.sigmoid(a)

# Actually Optimize
def optimize(model, data):
	optimizer = Adam(model.parameters(), lr=0.001)
	x = makeInputTensor(data)
	y = makeOutputTensor(data)
	bestParams = None
	minLoss = float("inf")
	while True:
		pred = model(x)
		loss = F.binary_cross_entropy(pred, y)
		loss.backward()
		optimizer.step()
		currParams = [model.theta1.item(), model.theta0.item()]
		if loss.item() < minLoss:
			bestParams = currParams
			minLoss = loss.item()
		output(loss.item(), bestParams, currParams)


########### Helper methods #########

def output(loss, bestParams, currParams):
	s = 'loss = {:.4f}, c1 = {:.4f}, c2 = {:.4f}'.format(
		loss, 
		bestParams[0], 
		bestParams[1]
	)
	print(s)

# Torch is very specific that the input has to be a list of matrices
# Our current input is just a single value, so we are going to need
# to make it a n x 1 "tensor".
def makeInputTensor(data):
	values = data[:,0]
	return torch.tensor(values).unsqueeze(1).float()

# Torch is very specific that the out has to be a list of matrices
# Our current output is just a single value, so we are going to need
# to make it a n x 1 "tensor".
def makeOutputTensor(data):
	values = data[:,1]
	return torch.tensor(values).unsqueeze(1).float()

def loadData():
	return np.genfromtxt('logRegData.csv', delimiter=',')


if __name__ == '__main__':
	main()