import numpy as np
import torch
import torch.nn as nn
from torch.optim import Adam
from torch.optim import SGD
import torch.nn.functional as F
import math

'''
This file is an example of using Torch to optimize.
Be careful, while correct, this version is *slow*
'''
def main():
	data = loadData()
	data = data[1:2000] # use only some data
	model = LogRegressionModel()
	optimizeBasic(model, data)

# 1) define the model you are optimizing over...

# This is a component of a neural net or any 
# parameterized function you are optimizing over. 
class LogRegressionModel(nn.Module):
	# the initialize method
	def __init__(self):
		super().__init__() # necessary
		# define any parameters are part of the model
		initial1 = torch.ones(1) * 0
		initial2 = torch.ones(1) * 0
		self.theta1 = nn.Parameter(initial1)
		self.theta0 = nn.Parameter(initial2)

	# define 
	def forward(self, x):
		a = self.theta1 * x + self.theta0
		return torch.sigmoid(a)

# 2) Actually optimize
def optimizeBasic(model, data):
	optimizer = Adam(model.parameters(), lr=0.005)
	while True:

		# In this example I am going to calculate the loss one 
		# data point at a time. It turns out that this is terribly
		# slow....
		loss = 0
		for i in range(len(data)):
			x, r = data[i]
			x = valueToTensor(x)
			r = valueToTensor(r)
			pred = model(x)
			loss_i = F.binary_cross_entropy(pred, r)
			loss += loss_i
		loss /= len(data)
		loss.backward()
		optimizer.step()
		print(loss.item(),i, model.theta1.item(), model.theta0.item())

########### Helper methods #########

# Torch is very specific that the input has to be a list of matrices
# Our current input is just a single value, so we are going to need
# to make it a n x 1 "tensor".
def valueToTensor(v):
	return torch.tensor([[v]]).float()

def loadData():
	return np.genfromtxt('logRegData.csv', delimiter=',')


if __name__ == '__main__':
	main()