import numpy as np
import torch
import torch.nn as nn
from torch.optim import Adam
from torch.optim import SGD
import torch.nn.functional as F
import math

def main():
	data = loadData()
	model = ExampleModel()
	optimize(model, data)

# 1) define the model you are optimizing over...

# This is a component of a neural net or any 
# parameterized function you are optimizing over. 
class MyModel(nn.Module):
	# the initialize method
	def __init__(self):
		super().__init__() # necessary

		# define any parameters you want
		self.myParam = nn.Parameter(torch.ones(1))

	# define the model function
	def forward(self, x):
		# encode the function using params and
		return 0

# 2) Actually optimize
def optimize(model, data):
	optimizer = Adam(model.parameters(), lr=0.001)
	x = makeInputTensor(data)
	y = makeOutputTensor(data)
	bestParams = None
	minLoss = float("inf")
	while True:
		pred = model(x)
		loss = F.binary_cross_entropy(pred, y)
		loss.backward()
		optimizer.step()
		currParams = [model.coeff1.item(), model.coeff2.item()]
		if loss.item() < minLoss:
			bestParams = currParams
			minLoss = loss.item()
		output(loss.item(), bestParams, currParams)


########### Helper methods #########

def output(loss, bestParams, currParams):
	s = 'loss = {:.4f}, c1 = {:.4f}, c2 = {:.4f}'.format(
		loss, 
		bestParams[0], 
		bestParams[1]
	)
	print(s)

def makeInputTensor(data):
	values = data[:,0]
	return torch.tensor([[values]]).float()

def makeOutputTensor(data):
	values = data[:,1]
	return torch.tensor([[values]]).float()

def valueToTensor(v):
	return torch.tensor([[[v]]]).float()

def loadData():
	return np.genfromtxt('data.csv', delimiter=',')


if __name__ == '__main__':
	main()